//
// Created by gordon on 2024/5/31.
//

#ifndef ALIBABACLOUD_RUM_H
#define ALIBABACLOUD_RUM_H

#include <stdarg.h>
#include <stddef.h>
#include <inttypes.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef ALIBABACLOUD_RUM_NATIVE_NAME
#   define  ALIBABACLOUD_RUM_NATIVE_NAME "alibabacloud.rum.native"
#endif

//    SDK Version
#define ALIBABACLOUD_RUM_NATIVE_SDK_NAME "alibabacloud_rum"
#define ALIBABACLOUD_RUM_NATIVE_VERSION "0.1.0"
#define ALIBABACLOUD_RUM_NATIVE_USER_AGENT ALIBABACLOUD_RUM_NATIVE_SDK_NAME "/" ALIBABACLOUD_RUM_NATIVE_VERSION

#ifdef _WIN32
#   define ALIBABACLOUD_RUM_PLATFORM_WINDOWS
#elif defined(__APPLE__)

#   include <TargetConditionals.h>

#   if defined(TARGET_OS_OSX) && TARGET_OS_OSX
#       define ALIBABACLOUD_RUM_PLATFORM_MACOS
#   elif defined(TARGET_OS_IPHONE) && TARGET_OS_IPHONE
#       define ALIBABACLOUD_RUM_PLATFORM_IOS
#   endif
#   define ALIBABACLOUD_RUM_PLATFORM_DARWIN
#   define ALIBABACLOUD_RUM_PLATFORM_UNIX
#elif defined(__LINUX) || defined(__inux__)
#   define ALIBABACLOUD_RUM_PLATFORM_LINUX
#   define ALIBABACLOUD_RUM_PLATFORM_UNIX
#else
#   error unsupported platform
#endif

/* marks a function as part of the AlibabacloudRUM API*/
#ifndef ALIBABACLOUD_RUM_API
#   ifdef  _WIN32
#       if defined(ALIBABACLOUD_RUM_BUILD_SHARED)
#           define ALIBABACLOUD_RUM_API __declspec(dllexport)
#       elif !defined(ALIBABACLOUD_RUM_BUILD_STATIC)
#           define ALIBABACLOUD_RUM_API __declspec(dllimport)
#       else
#           define ALIBABACLOUD_RUM_API
#       endif
#   else
#       if __GNUC__ >= 4
#           define ALIBABACLOUD_RUM_API __attribute__((visibility("default")))
#       else
#           define ALIBABACLOUD_RUM_API
#       endif
#   endif
#endif

#ifdef _WIN32
#   include <windows.h>
#else

#   include <signal.h>

#endif

ALIBABACLOUD_RUM_API void *alibabacloud_rum_malloc(size_t size);

ALIBABACLOUD_RUM_API void *alibabacloud_rum_realloc(void* ptr, size_t size);

ALIBABACLOUD_RUM_API void alibabacloud_rum_free(void *ptr);

/**
 * The AlibabaCloud RUM Log Level.
 */
typedef enum alibabacloud_rum_level_e
{
    ALIBABACLOUD_RUM_LEVEL_FATAL = 1,
    ALIBABACLOUD_RUM_LEVEL_ERROR = 2,
    ALIBABACLOUD_RUM_LEVEL_WARNING = 3,
    ALIBABACLOUD_RUM_LEVEL_INFO = 4,
    ALIBABACLOUD_RUM_LEVEL_DEBUG = 5,
    ALIBABACLOUD_RUM_LEVEL_TRACE = 6,
    ALIBABACLOUD_RUM_LEVEL_ALL = 7,
} alibabacloud_rum_level_t;

/**
 * The AlibabaCloud RUM logger function.
 */
typedef void(*alibabacloud_rum_logger_function_t)(
        alibabacloud_rum_level_t level, const char *message, va_list args, void *userdata
);

/**
 * The AlibabaCloud RUM Options.
 */
struct alibabacloud_rum_options_s;
typedef struct alibabacloud_rum_options_s alibabacloud_rum_options_t;

/**
 * The AlibabaCloud RUM Options constructor.
 * @return
 */
ALIBABACLOUD_RUM_API alibabacloud_rum_options_t *alibabacloud_rum_options_new(void);

ALIBABACLOUD_RUM_API void alibabacloud_rum_options_free(alibabacloud_rum_options_t *);

/**
 * Set the AlibabaCloud RUM Config Address.
 * @param options
 * @param config_address
 */
ALIBABACLOUD_RUM_API void alibabacloud_rum_options_set_config_address(
        alibabacloud_rum_options_t *options,
        const char *config_address
);

/**
 * Set the AlibabaCloud RUM Application Id.
 * @param options
 * @param app_id
 */
ALIBABACLOUD_RUM_API void alibabacloud_rum_options_set_app_id(
        alibabacloud_rum_options_t *options,
        const char *app_id
);

ALIBABACLOUD_RUM_API void alibabacloud_rum_options_set_env(alibabacloud_rum_options_t *options, const char *env);

ALIBABACLOUD_RUM_API void alibabacloud_rum_options_set_app_version(
        alibabacloud_rum_options_t *options,
        const char *app_version);

ALIBABACLOUD_RUM_API void alibabacloud_rum_options_set_debug_level(alibabacloud_rum_options_t *options, alibabacloud_rum_level_t level);

ALIBABACLOUD_RUM_API void alibabacloud_rum_set_username(const char *username);

/**
 * The AlibabaCloud RUM initialization method.
 * @param options
 * @return
 */
ALIBABACLOUD_RUM_API int alibabacloud_rum_init(alibabacloud_rum_options_t *options);

ALIBABACLOUD_RUM_API int alibabacloud_rum_close(void);

ALIBABACLOUD_RUM_API void alibabacloud_rum_start_session(void);

ALIBABACLOUD_RUM_API void alibabacloud_rum_end_session(void);

// custom event start
struct alibabacloud_rum_event_s;
typedef struct alibabacloud_rum_event_s alibabacloud_rum_event_t;

ALIBABACLOUD_RUM_API alibabacloud_rum_event_t* alibabacloud_rum_event_new(char *type, char *name);
ALIBABACLOUD_RUM_API void alibabacloud_rum_event_set_value(alibabacloud_rum_event_t *event, double value);
ALIBABACLOUD_RUM_API void alibabacloud_rum_event_set_group(alibabacloud_rum_event_t *event, char *group);
ALIBABACLOUD_RUM_API void alibabacloud_rum_event_add_custom(alibabacloud_rum_event_t *event, char *k, char *v);
ALIBABACLOUD_RUM_API void alibabacloud_rum_event_free(alibabacloud_rum_event_t *event);
ALIBABACLOUD_RUM_API void alibabacloud_rum_report_event(alibabacloud_rum_event_t *event);
// custom event end

// uuid start
typedef struct alibabacloud_rum_uuid_s {
    char bytes[16];
} alibabacloud_rum_uuid_t;

alibabacloud_rum_uuid_t alibabacloud_rum_uuid_new(void);
alibabacloud_rum_uuid_t alibabacloud_rum_uuid_nil(void);
alibabacloud_rum_uuid_t alibabacloud_rum_uuid_from_bytes(const char bytes[16]);
void alibabacloud_rum_uuid_as_string(alibabacloud_rum_uuid_t *uuid, char string[37]);
// uuid end
#ifdef __cplusplus
}
#endif
#endif //ALIBABACLOUD_RUM_H
